//*************************************************************************************************//
// Titolo: 			Servo Controller v1.0
//					(tratto da un lavoro di Samuel Oliver Wane 	http://www.fcet.staffs.ac.uk/sow1)
//					
// Autore:			Marcello Pinna	20/06/2012
// 
// Descrizione: 	Controllo e pilotaggio di un servocomando tramite interfaccia grafica
// 
// Hardware:		Freedom I 
//					pic 18F4550
//					quarzo 20 MHz
//					servocomando Futaba S3003
//
// Note importanti: il servo deve essere collegato a PORTD.RD0 del pic 18F4550 ed alimentato con
//					una tensione continua di 6 volts. Il codice  stato scritto seguendo le linee
//					guida di Mauro Laurenti, al quale va il mio personale ringraziamento. 
//*************************************************************************************************//

#include <p18F4550.h>
#include <usart.h>

#define SERVO_ON LATDbits.LATD0 = 1
#define SERVO_OFF LATDbits.LATD0 = 0

#define FACTOR 500				// fattore di conversione conteggi - mS

#pragma config FOSC = HS 		//OSC    = HS  	Impostato per lavorare ad alta frequenza 	
#pragma config WDT = OFF 		//WDT    = OFF	Disabilito il watchdog timer
#pragma config LVP = OFF 		//LVP    = OFF 	Disabilito programmazione LVP	
#pragma config PBADEN = OFF		//PBADEN = OFF  Disabilito gli ingressi analogici

// ============================= VARIABILI DI CONTROLLO ==================================
	
unsigned int timer0; 			// vedi timers.h
unsigned int timer1; 			// vedi timers.h


unsigned char data[2];			// array che contiene il dato in uscita		
unsigned char tamp0;			// variabile tampone
unsigned char tamp1;			// variabile tampone
unsigned char ms = 15;			// posizione di partenza del servo (centrale)
// ============================= PROTOTIPI DI FUNZIONE ===================================

void WriteTimer0 ( unsigned int timer0); // Prototipo di funzione estrapolato dal timers.h (MCC18\h)
void WriteTimer1 ( unsigned int timer1); // Prototipo di funzione estrapolato dal timers.h (MCC18\h)

// ======================== IMPOSTAZIONI TIMER0 E TIMER1 =================================

void set_timer(void) {

	// ---> TIMER0
	// Imposto la modalit a 16 bit
	T0CONbits.T08BIT = 0;
	
	// Imposto il clock interno
	T0CONbits.T0CS = 0;

	// Disabilito Prescaler 
	T0CONbits.PSA = 1;

	// Abilito le interruzioni del Timer0
	INTCONbits.TMR0IE = 1;

	// Abilito le interruzioni del Timer0 ad alta priorit
	INTCON2bits.TMR0IP = 1;

	// Abilito il Timer0
	T0CONbits.TMR0ON = 1;
	
	//---> TIMER1
	// Imposto la modalit a 16 bit
	T1CONbits.RD16 = 1;
		
	// Abilito il clock interno Timer1  (Fosc/4)
	T1CONbits.TMR1CS = 0; 
	
	// Imposto il PreScaler Timer1 1:4
	T1CONbits.T1CKPS0 = 0;
	T1CONbits.T1CKPS0 = 1;
	
	// Disabilito l'oscillatore del Timer1 
	T1CONbits.T1OSCEN = 0;
	
	// Abilito il Timer1
	T1CONbits.TMR1ON = 1;
	
	// Calcolo del valore iniziale del Timer1
	// Voglio generare un impulso della durata totale di 20 ms; con il prescaler impostato 1:4
	// il tempo di inremento  di 0.0004 ms. Il numero dei conteggi  dato da (20/0.0004 = 50000)
	// Sottraendo a 65536 il numero dei conteggi cos ottenuti si ha: (65536-50000 = 15536)
	timer1 = 15536;
	
	// Imposto la funzione WriteTimer1 con questo valore
	WriteTimer1 (timer1);
		
	// Abilito l'interrupt globale
	INTCONbits.GIEH = 1; 
}

// =========================== ROUTINE INTERRUPT ALTA PRIORITA' =========================

// Prototipo di funzione
void Hight_Int_Event (void);
 
#pragma code Hight_vector=0x08

void Hight_interrupt (void) {
	
	// Imposta il salto per la gestione dell'interrupt
	_asm GOTO Hight_Int_Event _endasm 
}

#pragma code

#pragma interrupt Hight_Int_Event

void Hight_Int_Event (void) {

	if (INTCONbits.TMR0IF) {
	
	// Spengo il servo
	SERVO_OFF; 
	
	// Disabilito il Timer0
	T0CONbits.TMR0ON = 0; 
	}
	
	// Appena scatta l'interrupt
	if (PIR1bits.TMR1IF) {
	
	// Ricarico il Timer1
	WriteTimer1(15536);
	
	// Azzero il flag di interrupt del timer1
    PIR1bits.TMR1IF = 0;
	
	// Avvio il servo
	SERVO_ON;
	
	// Converto i ms in numero di conteggi da levare a 65536 per ottenere il valore
	// iniziale del Timer0
	timer0 = (unsigned int)(65536 - (ms * FACTOR));
	
	// Imposto la funzione WriteTimer0 con questo valore
	WriteTimer0 (timer0);

	// Abilito il Timer0
    T0CONbits.TMR0ON = 1;
	
	// Pulisco il flag di interrupt
	INTCONbits.TMR0IF = 0;
 	}
}

// ==================================== ROUTINE PRINCIPALE ==============================

void main (void) {

	// Imposto TRISA tutti ingressi 
	LATA = 0x00; 
 	TRISA = 0xFF;

	// Imposto TRISB tutti ingressi
 	LATB = 0x00; 
 	TRISB = 0xFF;

	// Imposto TRISC tutti ingressi
 	LATC = 0x00;    	 
 	TRISC = 0xFF;
 	
	// Imposto RD0 come uscita
 	LATD = 0x00;  
 	TRISD = 0xFE;

	// Imposto TRISE tutti ingressi
 	LATE = 0x00;
 	TRISE = 0xFF;

	// Configura l'USART
 	// 8 bit
 	// 19200 bit/s
 	// 1 bit stop
 	// 0 bit parit

 	OpenUSART( USART_TX_INT_OFF &
 	USART_RX_INT_OFF &
 	USART_ASYNCH_MODE &
 	USART_EIGHT_BIT &
 	USART_CONT_RX &
 	USART_BRGH_HIGH,
 	64 );

	// Avvio la routine di inizializzazione
	set_timer();

	// Ciclo infinito
 	while(1) {

	// Attendo di ricevere un dato dal PC
 	while(!DataRdyUSART( ));

 	// Leggo il primo dato dal buffer di ricezione e le metto nell'array
 	data[0] = ReadUSART();
	
	// Attendo di ricevere un dato dal PC
 	while(!DataRdyUSART( ));
	
	// Leggo il secondo dato dal buffer di ricezione e le metto nell'array
	data[1] = ReadUSART();
	

	// Verifico se dall'interfaccia grafica ricevo la stringa "RX"; in caso affermativo...
	if ((data[0]) == 'R' && (data[1]) == 'X') {
	
	// Invio la conferma di avvenuta connessione al terminale
 	putrsUSART ("Freedom System connected to the serial port ");
	}
	
	else {
		
	// Invio il carattere al terminale
 	WriteUSART (data[0]);
 		
 	// Attendo che il dato venga trasmesso
 	while (BusyUSART());
	
	// Invio il carattere al terminale
 	WriteUSART (data[1]);
 		
 	// Attendo che il dato venga trasmesso
 	while (BusyUSART());
	
	// Inserisco uno spazio tra un numero e l'altro
	putrsUSART (" ");
	
	// Attendo che il dato venga trasmesso
 	while (BusyUSART());

	// Converto le singole cifre del dato da ASCII in numero, usando le variabili tampone 
	tamp0 = data[0] - 48;
	tamp1 = data[1] - 48;
	
	// Ricostruisco il dato originale
	ms = (tamp0 * 10) + tamp1;
	}		
	
  }
}